<?php
/** 
* @file rsvp_views.module
* @brief Views integration for RSVP module
* @author  Omar Abdel-Wahab
* @date 2007-08-31
 */

/** 
* Declare rsvp tables to views.
* 
* @return array of tables.
 */
function rsvp_views_tables() {
  $tables['rsvp'] = array(
    'name' => 'rsvp',
    'provider' => 'rsvp',
    'join' => array(
      'type' => 'inner',
      'left' => array(
        'table' => 'node',
        'field' => 'nid'
      ),
      'right' => array(
        'field' => 'nid'
      )
    ),
    'fields' => array(
      'total_rsvps' => array(
        'name' => t('RSVP: Total RSVPs'),
        'sortable' => TRUE,
        'notafield' => TRUE,
        'query_handler' => 'views_query_handler_field_rsvp_total_rsvps',
        'help' => t('Display total RSVPs for the node.'),
      ),
    ),
    'sorts' => array(
      'total_rsvp' => array(
        'name' => t('RSVP: Total RSVPs'),
        'handler' => 'views_rsvp_sort_handler_total_rsvp',
        'help' => t('Sort by the total RSVPs on a node'),
      ),
    ),
  );
  $tables['rsvp_invite'] = array(
    'name' => 'rsvp_invite',
    'provider' => 'rsvp',
    'join' => array(
      'left' => array(
        'table' => 'rsvp',
        'field' => 'rid'
      ),
      'right' => array(
        'field' => 'rid'
      )
    ),
    'fields' => array(
      'total_invites' => array(
        'name' => t('RSVP: Total Invites'),
        'sortable' => TRUE,
        'notafield' => TRUE,
        'query_handler' => 'views_query_handler_field_rsvp_total_invites',
        'help' => t('Display total invites for all RSVPs on the node.'),
      ),
    ),
    'sorts' => array(
      'total_invites' => array(
        'name' => t('RSVP: Total Invites'),
        'help' => t('Sort by the total invites in all RSVPs on a node'),
      ),
    ),
  );
  $tables['users'] = array(
    'name' => 'users',
    'provider' => 'internal',
    'join' => array(
      'left' => array(
        'table' => 'rsvp_invite',
        'field' => 'uid'
      ),
      'right' => array(
        'field' => 'uid'
      )
    ),
    'fields' => array(
      'uid' => array(
        'name' => t('RSVP: UID of Confirmed Users'),
        'sortable' => TRUE,
        'notafield' => FALSE,
        'query_handler' => 'views_query_handler_field_rsvp_users_confirmed',
        'help' => t('Display UID of users who confirmed attending for any RSVP on the node.'),
      ),
      'name' => array(
        'name' => t('RSVP: Username of Confirmed Users'),
        'addlfields' => array('uid'),
        'handler' => 'views_query_handler_field_rsvp_users_confirmed_name_link',
        'option' => array(
          '#type' => 'select',
          '#options' => array(
            'link' => 'As link',
            'nolink' => 'Without link',
          ),
        ),
        'sortable' => TRUE,
        'notafield' => FALSE,
        'query_handler' => 'views_query_handler_field_rsvp_users_confirmed',
        'help' => t('Display username of users who confirmed attending for any RSVP on the node.'),
      ),
    ),
  );

  return $tables;
}

/**
 * Handler for total RSVPs field.
 */
function views_query_handler_field_rsvp_total_rsvps($fielddata, $fieldinfo, &$query) {
  $query->ensure_table('rsvp');
  $query->add_field('COUNT(DISTINCT rsvp.rid)', '', $fielddata['tablename'] . '_total_rsvps');
  $query->groupby[] = 'rsvp.nid';
  $query->count_field = 'rsvp.nid';
}

/**
 * Handler for total invites field.
 */
function views_query_handler_field_rsvp_total_invites($fielddata, $fieldinfo, &$query) {
  $query->ensure_table('rsvp_invite');
  $query->add_field('COUNT(rsvp_invite.hash)', '', $fielddata['tablename'] . '_total_invites');
  $query->groupby[] = 'rsvp.nid';
}

/**
 * Handler for user confirmed attending.
 */
function views_query_handler_field_rsvp_users_confirmed($fielddata, $fieldinfo, &$query) {
  $query->add_where('rsvp_invite.response = "yes"');
}

function views_query_handler_field_rsvp_users_confirmed_name_link($fieldinfo, $fielddata, $value, $data) {
  if ($fielddata['options'] == 'nolink') {
    $link = check_plain($value);
  }
  else {
    $link = l($value, "user/$data->users_uid");
  }
  return $link;
}

function views_rsvp_sort_handler_total_rsvp($action, &$query, $sortinfo, $sort) {
  $query->orderby[] = "rsvp.rsvp_total_rsvps $sort[sortorder]";
}

function rsvp_views_default_views() {
  $view = new stdClass();
  $view->name = 'rsvp_attending';
  $view->description = 'RSVP: Users Attending';
  $view->access = array (
);
  $view->view_args_php = '$args[0] = arg(1);
return $args;';
  $view->page = FALSE;
  $view->page_title = 'RSVP: Users Attending';
  $view->page_header = '';
  $view->page_header_format = '1';
  $view->page_footer = '';
  $view->page_footer_format = '1';
  $view->page_empty = '';
  $view->page_empty_format = '1';
  $view->page_type = 'table';
  $view->url = '';
  $view->use_pager = FALSE;
  $view->nodes_per_page = '10';
  $view->block = TRUE;
  $view->block_title = 'RSVP: Users Attending';
  $view->block_header = '';
  $view->block_header_format = '1';
  $view->block_footer = '';
  $view->block_footer_format = '1';
  $view->block_empty = 'None';
  $view->block_empty_format = '1';
  $view->block_type = 'list';
  $view->nodes_per_block = '20';
  $view->block_more = TRUE;
  $view->block_use_page_header = FALSE;
  $view->block_use_page_footer = FALSE;
  $view->block_use_page_empty = FALSE;
  $view->sort = array (
  );
  $view->argument = array (
    array (
      'type' => 'nid',
      'argdefault' => '7',
      'title' => '',
      'options' => '0',
      'wildcard' => '',
      'wildcard_substitution' => '',
    ),
  );
  $view->field = array (
    array (
      'tablename' => 'users',
      'field' => 'name',
      'label' => '',
      'options' => 'link',
    ),
  );
  $view->filter = array (
    array (
      'tablename' => 'node',
      'field' => 'status',
      'operator' => '=',
      'options' => '',
      'value' => '1',
    ),
  );
  $view->exposed_filter = array (
  );
  $view->requires = array(users, node);
  $views[$view->name] = $view;

  return $views;
}
